import 'package:enx_uikit_flutter/controller/enx_controller.dart';
import 'package:enx_uikit_flutter/utilities/color.dart';
import 'package:enx_uikit_flutter/utilities/enx_setting.dart';
import 'package:flutter/material.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_instance/src/extension_instance.dart';
import 'package:get/get_state_manager/src/rx_flutter/rx_obx_widget.dart';

class CustomAppBar extends StatelessWidget implements PreferredSizeWidget {
  final double height;
  final String appMode;
  final bool role;
  final int floorAction;
  EnxController enxController = Get.find<EnxController>();
  final void Function() participantsCallbacks;

  final void Function(String map) moreCallbacks;
  final void Function() raiseHandCallbacks;
  List<Widget> buttonsEnabled = [];
  CustomAppBar(
      {Key? key,
      required this.appMode,
      required this.role,
      required this.floorAction,
      this.height = kToolbarHeight,
      required this.participantsCallbacks,
      required this.moreCallbacks,
      required this.raiseHandCallbacks})
      : super(key: key);
  @override
  Size get preferredSize => Size.fromHeight(height);

  @override
  Widget build(BuildContext context) {
    Map<TopOption, Widget> buttonTopMap = <TopOption, Widget>{};
    var setting = EnxSetting.instance;

    if (setting.topList.isNotEmpty) {
      for (var element in setting.topList) {
        if (element == TopOption.userList) {
          buttonTopMap.putIfAbsent(TopOption.userList, () => _userList());
        } else if (element == TopOption.menu) {
          buttonTopMap.putIfAbsent(TopOption.menu, () => _menuButton());
        } else if (element == TopOption.requestFloor) {
          if (appMode.toUpperCase() != 'GROUP') {
            buttonTopMap.putIfAbsent(
                TopOption.requestFloor, () => raiseHandIcon());
          }
        }
      }
    } else {
      if (appMode.toUpperCase() == 'GROUP') {
        buttonTopMap = <TopOption, Widget>{
          TopOption.userList: _userList(),
          TopOption.menu: _menuButton(),
        };
      } else {
        buttonTopMap = <TopOption, Widget>{
          TopOption.userList: _userList(),
          TopOption.requestFloor: raiseHandIcon(),
          TopOption.menu: _menuButton(),
        };
      }
    }
    for (var i = 0; i < buttonTopMap.length; i++) {
      buttonsEnabled.add(buttonTopMap.values.toList()[i]);
    }

    return ConstrainedBox(
      constraints: BoxConstraints(maxWidth: buttonsEnabled.length * 60),
      child: Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(14.0),
          color: Colors.grey.withOpacity(1),
        ),
        height: preferredSize.height,
        alignment: Alignment.center,
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            for (var i = 0; i < buttonsEnabled.length; i++) buttonsEnabled[i],
          ],
        ),
      ),
    );
  }

  Widget raiseHandIcon() {
    print('appmode:$appMode');

    return Obx(() => Expanded(
        child: InkWell(
            onTap: () {
              raiseHandCallbacks();
            },
            child: Center(
              child: role
                  ? Icon(
                      Icons.notifications_on_outlined,
                      color: enxController.isNotification
                          ? CustomColors.themeColor
                          : Colors.white,
                    )
                  : Image.asset(
                      floorAction == 0
                          ? "assets/image/raise_hand.png"
                          : floorAction == 1
                              ? "assets/image/raise_hand_cancel.png"
                              : "assets/image/raise_hand_finish.png",
                      package: 'enx_uikit_flutter',
                      color: Colors.white,
                    ),
            ))));
  }

  Widget _menuButton() {
    return Expanded(
        child: Center(
            child: InkWell(
                onTap: () {
                  moreCallbacks('');
                },
                child: const Icon(
                  Icons.more_vert,
                  color: Colors.white,
                ))));
  }

  Widget _userList() {
    return Expanded(
        child: InkWell(
            onTap: () {
              participantsCallbacks();
            },
            child: const Center(
                child: Icon(
              Icons.supervisor_account,
              color: Colors.white,
            ))));
  }
}
