import 'package:enx_uikit_flutter/controller/enx_controller.dart';
import 'package:enx_uikit_flutter/models/ChatModel.dart';
import 'package:enx_uikit_flutter/utilities/utils.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/src/size_extension.dart';
import 'package:get/get.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';

class Avatar extends StatelessWidget {
  final double size;
  final image;
  final EdgeInsets margin;

  Avatar({this.image, this.size = 50, this.margin = const EdgeInsets.all(0)});

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: margin,
      child: Container(
        width: size,
        height: size,
        decoration: const BoxDecoration(
          shape: BoxShape.circle,
          image: DecorationImage(
            image: AssetImage(
              'assets/image/chat_icon_color.png',
              package: 'enx_uikit_flutter',
            ),
          ),
        ),
      ),
    );
  }
}

class ChatPage extends StatelessWidget {
  EnxController enxController;
  String type;

  String? clientId;

  TextEditingController msgCtr = TextEditingController();

  ChatPage(this.enxController, this.type, {Key? key, this.clientId})
      : super(key: key);
  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();
  final _scaffoldMessengerKey = GlobalKey<ScaffoldMessengerState>();
  String? _fileName;
  String? _saveAsFileName;
  List<PlatformFile>? _paths;
  String? _directoryPath;
  String? _extension;
  bool _isLoading = false;
  bool _userAborted = false;
  bool _multiPick = false;
  final FileType _pickingType = FileType.any;

  @override
  Widget build(BuildContext context) {
    print("12111$type");
    return Scaffold(
      backgroundColor: Colors.white,
      body: SafeArea(
        child: Obx(
          () => Stack(
            children: [
              Column(
                children: [
                  _topChat(),
                  _bodyChat(),
                  const SizedBox(
                    height: 120,
                  )
                ],
              ),
              _formChat(),
            ],
          ),
        ),
      ),
    );
  }

  List<ChatModel> getPrivateChat() {
    for (var chatData in enxController.privateChatListData) {
      if (chatData.clientId == clientId) {
        print("er12${chatData.chatList.length}");
        return chatData.chatList;
      }
    }
    return enxController.privateChatList;
  }

  Widget _bodyChat() {
    return Expanded(
      child: Container(
        padding: const EdgeInsets.only(left: 25, right: 25, top: 25),
        width: double.infinity,
        decoration: const BoxDecoration(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(20), topRight: Radius.circular(20)),
          color: Colors.white,
        ),
        child: ListView(
          controller: enxController.scrollController,
          shrinkWrap: true,
          physics: const ClampingScrollPhysics(),
          children: [
            for (var item
                in type == 'group' ? enxController.chatList : getPrivateChat())
              Column(
                crossAxisAlignment: CrossAxisAlignment.end,
                mainAxisAlignment: MainAxisAlignment.start,
                children: [
                  Row(
                    mainAxisAlignment: item.recieved!
                        ? MainAxisAlignment.end
                        : MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        '${item.recieved! ? item.sender : 'Me'}',
                        style: TextStyle(color: Colors.grey.shade400),
                      ),
                      Flexible(
                        child: Container(
                          margin: const EdgeInsets.only(
                              left: 2, right: 10, top: 10),
                          padding: const EdgeInsets.all(20),
                          decoration: BoxDecoration(
                            color: item.recieved!
                                ? Colors.indigo.shade100
                                : Colors.indigo.shade50,
                            borderRadius: item.broadcast
                                ? const BorderRadius.only(
                                    topLeft: Radius.circular(10),
                                    topRight: Radius.circular(10),
                                    bottomLeft: Radius.circular(10),
                                  )
                                : const BorderRadius.only(
                                    topLeft: Radius.circular(10),
                                    topRight: Radius.circular(10),
                                    bottomRight: Radius.circular(10),
                                  ),
                          ),
                          child:Column(children: [
                            Text('${item.message}'),
                            SizedBox(height: 6,),
                            Visibility(
                             visible: item.jsonData.isNotEmpty?true:false,
                             child: GestureDetector(
                                onTap: (){

                                  enxController.downloadFile(item.jsonData);
                                },
                                child: Align(
                                  alignment: Alignment.bottomLeft,
                                    child: Text('[ Download ]',style:TextStyle(color: Colors.blue, fontSize: 17.sp,fontWeight: FontWeight.bold),)),
                              ),
                           ),
                          ],) ,
                        ),
                      ),
                    ],
                  ),
                  Align(
                    alignment: item.recieved!
                        ? Alignment.bottomRight
                        : Alignment.bottomLeft,
                    child: Text(
                      '${item.currentTime}',
                      style: TextStyle(color: Colors.grey.shade400),
                      textAlign: TextAlign.justify,
                    ),
                  )
                ],
              )
          ],
        ),
      ),
    );
  }

  Widget _formChat() {
    return Positioned(
      child: Align(
        alignment: Alignment.bottomCenter,
        child: Container(
          height: 120,
          padding: const EdgeInsets.symmetric(vertical: 30, horizontal: 10),
          color: Colors.white,
          child: TextField(
            controller: msgCtr,
            decoration: InputDecoration(
              hintText: 'Type your message...',
              prefixIcon:  IconButton(
                alignment: Alignment.center,
                icon: const Icon(
                  Icons.attach_file_outlined,
                  color: Colors.red,
                  size: 30,
                ),
                onPressed: () {
                    print('ontape');
                    if (type == 'group') {
                      enxController.sendFile(true, [],type);
                    } else {
                       enxController.sendFile(false,  [clientId!],type);

                    }
                },
              ),
              suffixIcon: Container(
                decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(30), color: Colors.red),
                padding: const EdgeInsets.all(1),
                child: IconButton(
                  alignment: Alignment.center,
                  icon: const Icon(
                    Icons.send_rounded,
                    color: Colors.white,
                    size: 20,
                  ),
                  onPressed: () {
                    // msgCtr.text = "";
                    if (msgCtr.text.toString().isNotEmpty) {
                      if (type == 'group') {
                        enxController.sendMessage(
                            msgCtr.text.toString(), true, [], type);
                      } else {
                        enxController.sendMessage(
                            msgCtr.text.toString(), false, [clientId!], type);
                      }

                      print('ontape');
                      msgCtr.text = "";
                    } else {}
                  },
                ),
              ),
              filled: true,
              fillColor: Colors.blueGrey[50],
              labelStyle: const TextStyle(fontSize: 12),
              contentPadding: const EdgeInsets.all(20),
              enabledBorder: OutlineInputBorder(
                borderSide: const BorderSide(color: Colors.blueGrey),
                borderRadius: BorderRadius.circular(25),
              ),
              focusedBorder: OutlineInputBorder(
                borderSide: const BorderSide(color: Colors.blueGrey),
                borderRadius: BorderRadius.circular(25),
              ),
            ),
          ),
        ),
      ),
    );
  }

  _topChat() {
    return Container(
      padding: EdgeInsets.symmetric(horizontal: 30, vertical: 15),
      child: Row(
        children: [
          GestureDetector(
            onTap: () => Get.back(),
            child: Icon(
              Icons.arrow_back_ios,
              size: 25,
              color: Colors.black87,
            ),
          ),
          Text(
            type == 'group' ? 'Group Chat' : type,
            style: const TextStyle(
                fontSize: 20,
                fontWeight: FontWeight.bold,
                color: Colors.black87),
          ),
        ],
      ),
    );
  }


  // void _pickFiles() async {
  //   _resetState();
  //   try {
  //     _directoryPath = null;
  //     _paths = (await FilePicker.platform.pickFiles(
  //       type: _pickingType,
  //       allowMultiple: false,
  //       onFileLoading: (FilePickerStatus status) => print(status),
  //       allowedExtensions: (_extension?.isNotEmpty ?? false)
  //           ? _extension?.replaceAll(' ', '').split(',')
  //           : null,
  //     ))
  //         ?.files;
  //   } on PlatformException catch (e) {
  //     _logException('Unsupported operation' + e.toString());
  //   } catch (e) {
  //     _logException(e.toString());
  //   }
  //   // if (!mounted) return;
  //   // setState(() {
  //   //   _isLoading = false;
  //   //   _fileName =
  //   //   _paths != null ? _paths!.map((e) => e.name).toString() : '...';
  //   //   _userAborted = _paths == null;
  //   // });
  // }
  //
  // void _clearCachedFiles(BuildContext context) async {
  //   _resetState();
  //   try {
  //     bool? result = await FilePicker.platform.clearTemporaryFiles();
  //     ScaffoldMessenger.of(context).showSnackBar(
  //       SnackBar(
  //         backgroundColor: result! ? Colors.green : Colors.red,
  //         content: Text((result
  //             ? 'Temporary files removed with success.'
  //             : 'Failed to clean temporary files')),
  //       ),
  //     );
  //   } on PlatformException catch (e) {
  //     _logException('Unsupported operation' + e.toString());
  //   } catch (e) {
  //     _logException(e.toString());
  //   } finally {
  //     //setState(() => _isLoading = false);
  //   }
  // }
  //
  //
  // Future<void> _saveFile() async {
  //   _resetState();
  //   try {
  //     String? fileName = await FilePicker.platform.saveFile(
  //       allowedExtensions: (_extension?.isNotEmpty ?? false)
  //           ? _extension?.replaceAll(' ', '').split(',')
  //           : null,
  //       type: _pickingType,
  //     );
  //     /*setState(() {
  //       _saveAsFileName = fileName;
  //       _userAborted = fileName == null;
  //     });*/
  //   } on PlatformException catch (e) {
  //     _logException('Unsupported operation' + e.toString());
  //   } catch (e) {
  //     _logException(e.toString());
  //   } finally {
  //    // setState(() => _isLoading = false);
  //   }
  // }
  //
  // void _logException(String message) {
  //   print(message);
  //   _scaffoldMessengerKey.currentState?.hideCurrentSnackBar();
  //   _scaffoldMessengerKey.currentState?.showSnackBar(
  //     SnackBar(
  //       content: Text(message),
  //     ),
  //   );
  // }
  //
  // void _resetState() {
  //  /* if (!mounted) {
  //     return;
  //   }
  //   setState(() {
  //     _isLoading = true;
  //     _directoryPath = null;
  //     _fileName = null;
  //     _paths = null;
  //     _saveAsFileName = null;
  //     _userAborted = false;
  //   });*/
  // }
}
