import 'package:enx_uikit_flutter/buttons/custom_button.dart';
import 'package:enx_uikit_flutter/controller/enx_controller.dart';
import 'package:enx_uikit_flutter/models/user_model.dart';
import 'package:enx_uikit_flutter/utilities/color.dart';
import 'package:enx_uikit_flutter/utilities/enx_setting.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get_core/src/get_main.dart';
import 'package:get/get_instance/src/extension_instance.dart';
import 'package:get/get_navigation/src/extension_navigation.dart';
import 'package:get/get_navigation/src/routes/transitions_type.dart';
import 'package:get/get_state_manager/src/rx_flutter/rx_obx_widget.dart';
import 'chat_ui.dart';

class ParticipantScreen extends StatelessWidget {
  EnxController enxController = Get.find<EnxController>();
  bool isAudio = false, isVideo = false, isChat = false, isDisconnect = false;

  ParticipantScreen(this.enxController, {Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    if(kDebugMode) {
      print("1211${enxController.userList.length}");
    }
    Map<ParticipantListOption, Widget> buttonTopMap =
        <ParticipantListOption, Widget>{};
    var setting = EnxSetting.instance;
    print("212112${setting.participantListOption.length}");

    if (setting.participantListOption.isNotEmpty) {
      for (var element in setting.participantListOption) {
        if (element == ParticipantListOption.audio) {
          isAudio = true;
        } else if (element == ParticipantListOption.video) {
          isVideo = true;
        } else if (element == ParticipantListOption.chat) {
          isChat = true;
        } else if (element == ParticipantListOption.disconnect) {
          isDisconnect = true;
        }
      }
    }

    print("21211212$isDisconnect +");

    return Scaffold(
        body: SafeArea(
            child: InkWell(
      onTap: () {
        enxController.isShowLocalRemote.value = true;
        enxController.update();
        Get.back();
      },
      child: Obx(() => Column(
            children: [
              Container(
                  width: MediaQuery.of(context).size.width,
                  color: Colors.red,
                  child: Align(
                    alignment: Alignment.centerLeft,
                    child: ElevatedButton.icon(
                      onPressed: () {
                        enxController.update();
                        Get.back();
                        },
                      icon: const Icon(Icons.arrow_back),
                      label: Text(
                        'Moderators(${enxController.moderatorsList.length})',
                        style: const TextStyle(color: Colors.white),
                      ),
                      style: ElevatedButton.styleFrom(
                        elevation: 0,
                        backgroundColor: Colors.transparent,
                      ),
                    ),
                  ),),

              const Divider(),
              ListView.separated(
                shrinkWrap: true,
                // Let the ListView know how many items it needs to build.
                itemCount: enxController.moderatorsList.length,
                // Provide a builder function. This is where the magic happens.
                // Convert each item into a widget based on the type of item it is.
                itemBuilder: (context, index) {
                  final item = enxController.moderatorsList[index];

                  return ListTile(
                    leading: const Icon(Icons.person),
                    title: Text(
                        enxController.userModelSelf!.clientId == item.clientId
                            ? '${item.name!} (Me)'
                            : item.name!,
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                        style:
                            const TextStyle(color: Colors.black, fontSize: 13)),
                    trailing: participantButtonOption(context,item,setting.participantListOption,true),
                  );
                },
                separatorBuilder: (BuildContext context, int index) {
                  return const Divider();
                },
              ),
              Container(
                  width: MediaQuery.of(context).size.width,
                  color: Colors.red,
                  child: Padding(
                    padding: const EdgeInsets.all(12.0),
                    child: Text(
                      'Participants (${enxController.participantsList.length})',
                      style: const TextStyle(color: Colors.white),
                    ),
                  )),
              ListView.separated(
                shrinkWrap: true,
                // Let the ListView know how many items it needs to build.
                itemCount: enxController.participantsList.length,
                // Provide a builder function. This is where the magic happens.
                // Convert each item into a widget based on the type of item it is.
                itemBuilder: (context, index) {
                  final item = enxController.participantsList[index];

                  return ListTile(
                    leading: const Icon(Icons.person),
                    title: Text(
                        enxController.userModelSelf!.clientId == item.clientId
                            ? '${item.name!} (Me)'
                            : item.name!,
                        overflow: TextOverflow.ellipsis,
                        maxLines: 1,
                        style:
                            const TextStyle(color: Colors.black, fontSize: 13)),
                    trailing:participantButtonOption(context, item, setting.participantListOption,false),
                  );
                },
                separatorBuilder: (BuildContext context, int index) {
                  return const Divider();
                },
              ),
            ],
          )),
    )));
  }

  Widget participantButtonOption(BuildContext context, UserModel item, Set<ParticipantListOption> participantListOption, bool isMod){
  return participantListOption.isNotEmpty? Wrap(
      spacing: 8, // space between two icons
      children: <Widget>[
        isAudio?  CustomMaterialButton(
          onPressed: () => {
            if (enxController.userModelSelf!.clientId ==
                item.clientId)
              {enxController.toggleMuteAudio()}
            else
              {
                if (enxController.userModelSelf!.role !=
                    'participant')
                  {
                    if (item.isAudioMuted)
                      enxController
                          .toggleHardUnMuteAudio(item.clientId)
                    else
                      enxController
                          .toggleHardMuteAudio(item.clientId)
                  }
              }
          },
          child: Icon(
            item.isAudioMuted
                ? Icons.mic_off_outlined
                : Icons.mic_none_outlined,
            color: item.isAudioMuted
                ? CustomColors.themeColor
                : Colors.grey,
          ),
          elevation: 2.0,
          padding: const EdgeInsets.all(12.0),
        ):const SizedBox(), // icon-1
      isVideo?  CustomMaterialButton(
          onPressed: () => {
            if (enxController.userModelSelf!.clientId ==
                item.clientId)
              {enxController.toggleMuteVideo()}
            else
              {
                if (enxController.userModelSelf!.role !=
                    'participant')
                  {
                    if (item.isVideoMuted)
                      enxController
                          .toggleHardUnMuteVideo(item.clientId)
                    else
                      enxController
                          .toggleHardMuteVideo(item.clientId)
                  }
              }
          },
          child: Icon(
              item.isVideoMuted
                  ? Icons.videocam_off_outlined
                  : Icons.videocam_outlined,
              color: item.isVideoMuted
                  ? CustomColors.themeColor
                  : Colors.grey),
          elevation: 2.0,
          padding: EdgeInsets.all(12.0),
        ):const SizedBox(),
        enxController.userModelSelf!.clientId == item.clientId
            ? const SizedBox()
            : isChat?CustomMaterialButton(
          onPressed: () {
            enxController.chatType.value = 'private';
            Get.to(
                    () => ChatPage(
                  enxController,
                  item.name,
                  clientId: item.clientId,
                ),
                transition: Transition.leftToRight,
                duration: const Duration(seconds: 1));
          },
          child: Image.asset(
            "assets/image/chat_icon.png",
            package: 'enx_uikit_flutter',
            width: 30,
            height: 30,
          ),
          elevation: 2.0,
        ):const SizedBox(),
      isMod?  enxController.userModelSelf!.role == 'participant'
            ? const SizedBox()
            : isDisconnect?CustomMaterialButton(
          onPressed: () => _onCallEnd(
              context, [item.clientId],
              clientId: enxController
                  .userModelSelf!.clientId ==
                  item.clientId
                  ? enxController.userModelSelf!.clientId
                  : ''),
          child: const Icon(Icons.call_rounded,
              color: Colors.white, size: 20),
          shape: const CircleBorder(),
          elevation: 2.0,
          fillColor: CustomColors.themeColor,
        ):const SizedBox() :  (enxController.userModelSelf!.role == 'participant')
          ? enxController.userModelSelf!.clientId ==
          item.clientId
          ? isDisconnect?CustomMaterialButton(
        onPressed: () => _onCallEnd(
            context, [item.clientId],
            clientId: enxController
                .userModelSelf!.clientId),
        child: const Icon(Icons.call_rounded,
            color: Colors.white, size: 20),
        shape: const CircleBorder(),
        elevation: 2.0,
        fillColor: CustomColors.themeColor,
      ):const SizedBox()
          : const SizedBox()
          : enxController.userModelSelf!.clientId ==
          item.clientId
          ? const SizedBox()
          : isDisconnect?CustomMaterialButton(
        onPressed: () => _onCallEnd(
            context, [item.clientId],
            clientId: ''),
        child: const Icon(Icons.call_rounded,
            color: Colors.white, size: 20),
        shape: const CircleBorder(),
        elevation: 2.0,
        fillColor: CustomColors.themeColor,
      ):const SizedBox(),
      ],
    ):Wrap(
    spacing: 8, // space between two icons
    children: <Widget>[
      CustomMaterialButton(
        onPressed: () => {
          if (enxController.userModelSelf!.clientId ==
              item.clientId)
            {enxController.toggleMuteAudio()}
          else
            {
              if (enxController.userModelSelf!.role !=
                  'participant')
                {
                  if (item.isAudioMuted)
                    enxController
                        .toggleHardUnMuteAudio(item.clientId)
                  else
                    enxController
                        .toggleHardMuteAudio(item.clientId)
                }
            }
        },
        child: Icon(
          item.isAudioMuted
              ? Icons.mic_off_outlined
              : Icons.mic_none_outlined,
          color: item.isAudioMuted
              ? CustomColors.themeColor
              : Colors.grey,
        ),
        elevation: 2.0,
        padding: const EdgeInsets.all(12.0),
      ), // icon-1
      CustomMaterialButton(
        onPressed: () => {
          if (enxController.userModelSelf!.clientId ==
              item.clientId)
            {enxController.toggleMuteVideo()}
          else
            {
              if (enxController.userModelSelf!.role !=
                  'participant')
                {
                  if (item.isVideoMuted)
                    enxController
                        .toggleHardUnMuteVideo(item.clientId)
                  else
                    enxController
                        .toggleHardMuteVideo(item.clientId)
                }
            }
        },
        child: Icon(
            item.isVideoMuted
                ? Icons.videocam_off_outlined
                : Icons.videocam_outlined,
            color: item.isVideoMuted
                ? CustomColors.themeColor
                : Colors.grey),
        elevation: 2.0,
        padding: EdgeInsets.all(12.0),
      ),
      enxController.userModelSelf!.clientId == item.clientId
          ? const SizedBox()
          : CustomMaterialButton(
        onPressed: () {
          enxController.chatType.value = 'private';
          Get.to(
                  () => ChatPage(
                enxController,
                item.name,
                clientId: item.clientId,
              ),
              transition: Transition.leftToRight,
              duration: const Duration(seconds: 1));
        },
        child: Image.asset(
          "assets/image/chat_icon.png",
          package: 'enx_uikit_flutter',
          width: 30,
          height: 30,
        ),
        elevation: 2.0,
      ),
    isMod?  enxController.userModelSelf!.role == 'participant'
          ? const SizedBox()
          : CustomMaterialButton(
        onPressed: () => _onCallEnd(
            context, [item.clientId],
            clientId: enxController
                .userModelSelf!.clientId ==
                item.clientId
                ? enxController.userModelSelf!.clientId
                : ''),
        child: const Icon(Icons.call_rounded,
            color: Colors.white, size: 20),
        shape: const CircleBorder(),
        elevation: 2.0,
        fillColor: CustomColors.themeColor,
      ): (enxController.userModelSelf!.role == 'participant')
        ? enxController.userModelSelf!.clientId ==
        item.clientId
        ? CustomMaterialButton(
      onPressed: () => _onCallEnd(
          context, [item.clientId],
          clientId: enxController
              .userModelSelf!.clientId),
      child: const Icon(Icons.call_rounded,
          color: Colors.white, size: 20),
      shape: const CircleBorder(),
      elevation: 2.0,
      fillColor: CustomColors.themeColor,
    )
        : const SizedBox()
        : enxController.userModelSelf!.clientId ==
        item.clientId
        ? const SizedBox()
        : CustomMaterialButton(
      onPressed: () => _onCallEnd(
          context, [item.clientId],
          clientId: ''),
      child: const Icon(Icons.call_rounded,
          color: Colors.white, size: 20),
      shape: const CircleBorder(),
      elevation: 2.0,
      fillColor: CustomColors.themeColor,
    ),
    ],
  );

  }

  void _onCallEnd(BuildContext context, List<String> participants,
      {String? clientId}) {
    print('$clientId');
    if (clientId!.isNotEmpty) {
      enxController.endCall('participantScreen');
    } else {
      enxController.dropCall(participants);
    }
  }

/*
  participantButtonOptionParticipant(BuildContext context, UserModel item, participantListOption) {
    return  participantListOption.isNotEmpty?Wrap(
      spacing: 8, // space between two icons
      children: <Widget>[
       isAudio? CustomMaterialButton(
          onPressed: () => {
            if (enxController.userModelSelf!.clientId ==
                item.clientId)
              {enxController.toggleMuteAudio()}
            else
              {
                if (enxController.userModelSelf!.role !=
                    'participant')
                  {
                    if (item.isAudioMuted)
                      enxController
                          .toggleHardUnMuteAudio(item.clientId)
                    else
                      enxController
                          .toggleHardMuteAudio(item.clientId)
                  }
              }
          },
          child: Icon(
            item.isAudioMuted
                ? Icons.mic_off_outlined
                : Icons.mic_none_outlined,
            color: item.isAudioMuted
                ? CustomColors.themeColor
                : Colors.grey,
          ),
          elevation: 2.0,
          padding: const EdgeInsets.all(12.0),
        ):const SizedBox(), // icon-1
       isVideo? CustomMaterialButton(
          onPressed: () => {
            //
            if (enxController.userModelSelf!.clientId ==
                item.clientId)
              {enxController.toggleMuteVideo()}
            else
              {
                if (enxController.userModelSelf!.role !=
                    'participant')
                  {
                    if (item.isVideoMuted)
                      enxController
                          .toggleHardUnMuteVideo(item.clientId)
                    else
                      enxController
                          .toggleHardMuteVideo(item.clientId)
                  }
              }

            //
          },
          child: Icon(
              item.isVideoMuted
                  ? Icons.videocam_off_outlined
                  : Icons.videocam_outlined,
              color: item.isVideoMuted
                  ? CustomColors.themeColor
                  : Colors.grey),
          elevation: 2.0,
          padding: EdgeInsets.all(12.0),
        ):const SizedBox(),
        enxController.userModelSelf!.clientId == item.clientId
            ? const SizedBox()
            : isChat? CustomMaterialButton(
          onPressed: () {
            enxController.chatType.value = 'private';
            Get.to(
                    () => ChatPage(
                  enxController,
                  item.name,
                  clientId: item.clientId,
                ),
                transition: Transition.leftToRight,
                duration: const Duration(seconds: 1));
          },
          child: Image.asset(
            "assets/image/chat_icon.png",
            package: 'enx_uikit_flutter',
            width: 30,
            height: 30,
          ),
          elevation: 2.0,
        ):const SizedBox(),

        (enxController.userModelSelf!.role == 'participant')
            ? enxController.userModelSelf!.clientId ==
            item.clientId
            ? isDisconnect?CustomMaterialButton(
          onPressed: () => _onCallEnd(
              context, [item.clientId],
              clientId: enxController
                  .userModelSelf!.clientId),
          child: const Icon(Icons.call_rounded,
              color: Colors.white, size: 20),
          shape: const CircleBorder(),
          elevation: 2.0,
          fillColor: CustomColors.themeColor,
        ):const SizedBox()
            : const SizedBox()
            : enxController.userModelSelf!.clientId ==
            item.clientId
            ? const SizedBox()
            : isDisconnect?CustomMaterialButton(
          onPressed: () => _onCallEnd(
              context, [item.clientId],
              clientId: ''),
          child: const Icon(Icons.call_rounded,
              color: Colors.white, size: 20),
          shape: const CircleBorder(),
          elevation: 2.0,
          fillColor: CustomColors.themeColor,
        ):const SizedBox(), // icon-2
      ],
    ):Wrap(
      spacing: 8, // space between two icons
      children: <Widget>[
        CustomMaterialButton(
          onPressed: () => {
            if (enxController.userModelSelf!.clientId ==
                item.clientId)
              {enxController.toggleMuteAudio()}
            else
              {
                if (enxController.userModelSelf!.role !=
                    'participant')
                  {
                    if (item.isAudioMuted)
                      enxController
                          .toggleHardUnMuteAudio(item.clientId)
                    else
                      enxController
                          .toggleHardMuteAudio(item.clientId)
                  }
              }
          },
          child: Icon(
            item.isAudioMuted
                ? Icons.mic_off_outlined
                : Icons.mic_none_outlined,
            color: item.isAudioMuted
                ? CustomColors.themeColor
                : Colors.grey,
          ),
          elevation: 2.0,
          padding: EdgeInsets.all(12.0),
        ), // icon-1
        CustomMaterialButton(
          onPressed: () => {
            //
            if (enxController.userModelSelf!.clientId ==
                item.clientId)
              {enxController.toggleMuteVideo()}
            else
              {
                if (enxController.userModelSelf!.role !=
                    'participant')
                  {
                    if (item.isVideoMuted)
                      enxController
                          .toggleHardUnMuteVideo(item.clientId)
                    else
                      enxController
                          .toggleHardMuteVideo(item.clientId)
                  }
              }

            //
          },
          child: Icon(
              item.isVideoMuted
                  ? Icons.videocam_off_outlined
                  : Icons.videocam_outlined,
              color: item.isVideoMuted
                  ? CustomColors.themeColor
                  : Colors.grey),
          elevation: 2.0,
          padding: EdgeInsets.all(12.0),
        ),
        enxController.userModelSelf!.clientId == item.clientId
            ? const SizedBox()
            : CustomMaterialButton(
          onPressed: () {
            enxController.chatType.value = 'private';
            Get.to(
                    () => ChatPage(
                  enxController,
                  item.name,
                  clientId: item.clientId,
                ),
                transition: Transition.leftToRight,
                duration: const Duration(seconds: 1));
          },
          child: Image.asset(
            "assets/image/chat_icon.png",
            package: 'enx_uikit_flutter',
            width: 30,
            height: 30,
          ),
          elevation: 2.0,
        ),
        (enxController.userModelSelf!.role == 'participant')
            ? enxController.userModelSelf!.clientId ==
            item.clientId
            ? CustomMaterialButton(
          onPressed: () => _onCallEnd(
              context, [item.clientId],
              clientId: enxController
                  .userModelSelf!.clientId),
          child: const Icon(Icons.call_rounded,
              color: Colors.white, size: 20),
          shape: const CircleBorder(),
          elevation: 2.0,
          fillColor: CustomColors.themeColor,
        )
            : const SizedBox()
            : enxController.userModelSelf!.clientId ==
            item.clientId
            ? const SizedBox()
            : CustomMaterialButton(
          onPressed: () => _onCallEnd(
              context, [item.clientId],
              clientId: ''),
          child: const Icon(Icons.call_rounded,
              color: Colors.white, size: 20),
          shape: const CircleBorder(),
          elevation: 2.0,
          fillColor: CustomColors.themeColor,
        ), // icon-2
      ],
    );
  }
*/
}
